package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.KilnInfoQueryParam;
import com.mt.wms.basic.params.VehicleQueryParam;
import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.service.LocationInfoService;
import com.mt.wms.basic.service.VehicleService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.ApmsCheckResult;
import com.mt.wms.core.dal.entity.CurrTaskDet;
import com.mt.wms.core.dal.entity.OrderInfo;
import com.mt.wms.core.dal.service.ApmsCheckResultServiceBiz;
import com.mt.wms.core.dal.service.CommunicationLogServiceBiz;
import com.mt.wms.core.dal.service.OrderInfoServiceBiz;
import com.mt.wms.core.enums.WhetherEnum;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.controller.ApmsController;
import com.mt.wms.empty.enums.OrderSourceEnum;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.*;
import com.mt.wms.empty.vo.CurrTaskVo;
import com.mt.wms.empty.vo.OrderInfoTaskDetVo;
import com.mt.wms.empty.vo.OrderInfoVo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class OrderInfoServiceImpl extends BaseService implements OrderInfoService {

    @Autowired
    OrderInfoServiceBiz orderInfoService;

    @Autowired
    OrderInfoHisService orderInfoHisService;
    @Autowired
    CurrTaskDetService currTaskDetService;
    @Autowired
    TaskDetHisService taskDetHisService;
    @Autowired
    CurrTaskService currTaskService;
    @Autowired
    KilnInfoService kilnInfoService;

    @Autowired
    VehicleService vehicleService;

    @Autowired
    LocationInfoService locationInfoService;

    @Autowired
    CommunicationLogServiceBiz communicationLogServiceBiz;

    @Autowired
    ApmsCheckResultServiceBiz apmsCheckResultServiceBiz;
    @Autowired
    ApmsController apmsControl;

    @Override
    public R<String> apmsPostOrder(OrderParamForApms apmsPostOrderParam) {
        OrderInfo orderInfo = new OrderInfo();
        BeanUtils.copyProperties(apmsPostOrderParam, orderInfo);
        setCommonField(orderInfo);
        //设定来源信息为apms
        orderInfo.setOrderSource(OrderSourceEnum.APMS.getValue());
        orderInfo.setInterCode(IDGenerator.gen("DD", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "ORDER_CODE"));
        orderInfoService.save(orderInfo);
        return successful("Wms接收Apms订单成功。");
    }

    @Override
    public R<PageVo<OrderInfoVo>> page(OrderInfoBasePageParam param) {
        QueryWrapper<OrderInfo> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(param.getIdenCardNum()), OrderInfo.IDEN_CARD_NUM, param.getIdenCardNum())
                .like(StringUtils.isNotBlank(param.getOrderNo()), OrderInfo.ORDER_NO, param.getOrderNo())
                .eq(OrderInfo.VALID, 1)
                .orderByDesc(OrderInfo.CREATE_TIME);
        Page<OrderInfo> page = orderInfoService.page(new Page<>(param.getCurrent(), param.getSize()), wrapper);
        PageVo<OrderInfoVo> orderInfoVoPageVo = new PageVo<>(page, OrderInfoVo.class);
        orderInfoVoPageVo.getRecords().forEach(e ->
        {
            R<OrderInfoVo> info = getTaskInfoByIdenCardNum(e.getIdenCardNum());
            e.setFinishWeight(info.getData().getFinishWeight());
            e.setFinishQuantity(info.getData().getQuantity());
            e.setWaitWeight(info.getData().getWaitWeight());
            e.setWaitQuantity(info.getData().getWaitQuantity());
            e.setReProcessQuantity(info.getData().getReProcessQuantity());
            e.setReProcessWeight(info.getData().getReProcessWeight());
        });
        return successful(orderInfoVoPageVo);
    }

    @Override
    public R<String> personCreateOrder(OrderInfoPersonCreateParam orderInfoPersonCreateParam) {
        OrderInfo orderInfo = new OrderInfo();
        BeanUtils.copyProperties(orderInfoPersonCreateParam, orderInfo);
        setCommonField(orderInfo);
        //设定来源信息为人工
        orderInfo.setOrderSource(OrderSourceEnum.PERSON.getValue());
        orderInfo.setInterCode("PERSONAL" + System.currentTimeMillis());
        orderInfoService.save(orderInfo);
        return successful("人工创建订单成功。");
    }

    @Override
    public R<OrderInfoVo> getOne(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        OrderInfoVo build = OrderInfoVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        R<OrderInfoVo> info = getTaskInfoByIdenCardNum(build.getIdenCardNum());
        build.setFinishWeight(info.getData().getFinishWeight());
        build.setFinishQuantity(info.getData().getQuantity());
        build.setWaitWeight(info.getData().getWaitWeight());
        build.setWaitQuantity(info.getData().getWaitQuantity());
        build.setReProcessQuantity(info.getData().getReProcessQuantity());
        build.setReProcessWeight(info.getData().getReProcessWeight());
        return successful(build);
    }


    /**
     * 获取详细的拆分任务信息
      */
    @Override
    public R<PageVo<OrderInfoTaskDetVo>> getTaskInfo(OrderInfoTaskDetParam param) {
        //从任务详细中获取该标识卡和订单的所有订单
        PageVo<OrderInfoTaskDetVo> currOrderCurrTask = currTaskDetService.getCurrOrderCurrTask(param);
        Assert.gtZero(currOrderCurrTask.getRecords().size(),"该订单暂无详细记录。");
        //获取车辆信息
        R<List<VehicleVo>> vehicle = vehicleService.list(new VehicleQueryParam());
        Map<Long, List<VehicleVo>> collect = vehicle.getData().stream().collect(Collectors.groupingBy(VehicleVo::getId));
        //获取窑炉信息
        R<List<KilnInfoVo>> kiln = kilnInfoService.list(new KilnInfoQueryParam());
        Map<Long, List<KilnInfoVo>> collect2 = kiln.getData().stream().collect(Collectors.groupingBy(KilnInfoVo::getId));
        currOrderCurrTask.getRecords().forEach(orderInfo ->
                {
                    //获取对应的任务
                    CurrTaskVo currTask = currTaskService.getCurrTask(IdParam.builder().id(orderInfo.getCurrTaskId()).build());
                    orderInfo.setABeginTime(currTask.getABeginTime());
                    orderInfo.setAEndTime(currTask.getAEndTime());
                    orderInfo.setTaskType(currTask.getTaskType());
                    orderInfo.setStatus(currTask.getStatus());
                    orderInfo.setKilnId(currTask.getKilnId());
                    orderInfo.setStartPosition(currTask.getStartPosition());
                    orderInfo.setTargetPosition(currTask.getTargetPosition());
                    orderInfo.setVehicleId(currTask.getVehicleId());
                    //获取窑炉和车辆的名称信息
                    if (orderInfo.getKilnId() != null)
                    {
                        orderInfo.setKilnName(collect2.get(orderInfo.getKilnId()).get(0).getKilnName());
                    }
                    if (orderInfo.getVehicleId() != null)
                    {
                        orderInfo.setVehicleName(collect.get(orderInfo.getVehicleId()).get(0).getVehicleName());
                    }
                }
        );
        return successful(currOrderCurrTask);
    }

    @Override
    public R<OrderInfoVo> getTaskInfoByIdenCardNum(String idenCardNum) {
        QueryWrapper<OrderInfo> wrapper = new QueryWrapper<>();
        wrapper.eq(OrderInfo.IDEN_CARD_NUM, idenCardNum).eq(OrderInfo.VALID, 1);
        OrderInfo one = orderInfoService.getOne(wrapper);
        Assert.notNull("未找到该标识卡信息！", one);
        OrderInfoVo build = OrderInfoVo.builder().build();
        BeanUtils.copyProperties(one, build);
        //查询历史任务表，当前任务表，拉取加工完成 创建任务未加工 计算剩余数量
        HashMap<String, Double> sumCurrTask = currTaskDetService.getSumByIdCard(idenCardNum);
        build.setWaitQuantity(sumCurrTask.get("quantity").floatValue());
        build.setWaitWeight(sumCurrTask.get("weight").floatValue());

        HashMap<String, Double> sumTaskHis = taskDetHisService.getSumByIdCard(idenCardNum);
        build.setFinishQuantity(sumTaskHis.get("quantity").floatValue());
        build.setFinishWeight(sumTaskHis.get("weight").floatValue());

        // 考虑返工单的加工情况，添加到返工统计信息中。
        if (Objects.nonNull(one.getOldOrderNo()))
        {
            List<OrderInfo> list = orderInfoService.list(new QueryWrapper<OrderInfo>().eq(OrderInfo.VALID, 1).eq(OrderInfo.ORDER_NO, one.getOldOrderNo()));
            float reQuantity = 0;
            float reWeight = 0;
            for (OrderInfo orderInfo : list)
            {
                HashMap<String, Double> reCurrTask = currTaskDetService.getSumByIdCard(orderInfo.getIdenCardNum());
                reQuantity += reCurrTask.get("quantity").floatValue();
                reWeight += reCurrTask.get("weight").floatValue();

                HashMap<String, Double> reTaskHis = taskDetHisService.getSumByIdCard(orderInfo.getIdenCardNum());
                reQuantity += reTaskHis.get("quantity").floatValue();
                reWeight += reTaskHis.get("weight").floatValue();
            }
            build.setReProcessQuantity(reQuantity);
            build.setReProcessWeight(reWeight);
        }

        return successful(build);
    }

    @Override
    public void verifyTaskInfoByIdenCardNum(List<TaskCreateDetParam> detParams) {
        for (TaskCreateDetParam detParam : detParams)
        {
            QueryWrapper<OrderInfo> wrapper = new QueryWrapper<>();
            wrapper.eq(OrderInfo.IDEN_CARD_NUM, detParam.getIdenCardNum()).eq(OrderInfo.VALID, 1);
            OrderInfo one = orderInfoService.getOne(wrapper);
            Assert.notNull("错误，标识卡号" + detParam.getIdenCardNum() + "在订单中不存在!", one);
        }
    }

    @Override
    public R<String> completeOrder(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        byId.setValid(0);
        byId.setStatus(2);
        orderInfoService.updateById(byId);
        byId.setId(null);
        byId.setValid(1);
        byId.setInterCode(IDGenerator.gen("DDHIS", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "ORDER_HIS_CODE"));
        setCommonField(byId);
        orderInfoHisService.save(byId);
        return successful("成功，订单状态置为完成。");
    }

    @Override
    public R<String> voidOrder(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        byId.setUpdateTime(LocalDateTime.now());
        byId.setValid(0);
        orderInfoService.updateById(byId);
        return successful("成功，订单状态置为作废。");
    }

    @Override
    public R<String> apmsVoidOrder(String idenCardNum) {
        OrderInfo byId = orderInfoService.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, idenCardNum));
        //确定是否有订单在生产
        List<CurrTaskDet> taskDetailsByIdenCardNum = currTaskDetService.getTaskDetailsByIdenCardNum(idenCardNum);
        if (taskDetailsByIdenCardNum.size() > 0)
        {
            return failed("操作失败，已经存在生产中的任务单。");
        }
        else
        {
            return voidOrder(IdParam.builder().id(byId.getId()).build());
        }
    }

    @Override
    public R<String> apmsCompleteOrder(ApmsCompleteOrder apmsCompleteOrder) {
        //通过标识卡号与生产单号进行更新检验结果
        ApmsCheckResult one = new ApmsCheckResult();
        one.setIdenCardNum(apmsCompleteOrder.getIdenCardNum());
        one.setSheetNo(apmsCompleteOrder.getSheetNo());
        one.setCreateTime(LocalDateTime.now());
        one.setValid(WhetherEnum.YES.getValue());
        one.setMetallography(apmsCompleteOrder.getMetallographic());
        one.setHeartHardness(apmsCompleteOrder.getHeartHardness());
        one.setHardness(apmsCompleteOrder.getHardness());
        one.setUpdateTime(LocalDateTime.now());
        apmsCheckResultServiceBiz.save(one);
        //订单不应该在该地方完成，额外提供一个完成接口供人工完成，不通知apms
        return successful("任务执行完成。");
    }
}
