package com.mt.wms.empty.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.vo.R;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/15 21:55
 * @Version 1.0
 */
@Transactional
public class RunTaskUtils {
    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private AsynRunTaskService asynRunTaskService;
    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;

    /**
     * 进炉加工
     * @param currTaskId
     * @return
     * @throws InterruptedException
     */
    public R runTaskForInKiln(Integer currTaskId) throws InterruptedException {
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        //传入任务id,起始点，若终点是窑炉，获取窑炉状态
        int kilnStatus = 1;

        //窑炉状态为不可用
        if (kilnStatus == 0) {
            return R.failed("目标窑炉不可用！");
        }
        //窑炉可用未满
        if (kilnStatus==1){
            //更改任务状态为执行中
            currTask.setStatus(1);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //查询是否有空闲车辆，若有。占用车辆，若无，返回暂无可用车辆
            List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>()
                    .eq(VehicleInfo.STATUS, 0)
                    .eq(VehicleInfo.VALID, 1));
            if (vehicleInfoList.size()>0){
                VehicleInfo vehicleInfo = vehicleInfoList.get(0);
                vehicleInfo.setStatus(1);
                vehicleInfo.setUpdateTime(LocalDateTime.now());
                vehicleInfoServiceBiz.updateById(vehicleInfo);
            }else {
                return R.failed("暂无可用车辆!请稍后重试！");
            }
            //异步调用车辆
            // TODO: 2021/12/13 传入车辆id
            asynRunTaskService.asynRunTask(currTask);
            return R.ok("操作成功");
        }
        //窑炉已满
        if (kilnStatus==2){
            //查询缓存区库位是否已满
            int count = locationServiceBiz.count(new QueryWrapper<Location>()
                    .eq(Location.STATUS, 0)
                    .eq(Location.VALID, 1));
            //缓存区未满
            if (count >0) {
                //查询是否有空闲车辆，若有。占用车辆，若无，返回暂无可用车辆
                List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>()
                        .eq(VehicleInfo.STATUS, 0)
                        .eq(VehicleInfo.VALID, 1));
                if (vehicleInfoList.size()>0){
                    VehicleInfo vehicleInfo = vehicleInfoList.get(0);
                    vehicleInfo.setStatus(1);
                    vehicleInfo.setUpdateTime(LocalDateTime.now());
                    vehicleInfoServiceBiz.updateById(vehicleInfo);
                }else {
                    return R.failed("暂无可用车辆!请稍后重试！");
                }
                // TODO: 2021/12/13 更改 is_cache 、库位id、库位名称
                currTask.setUpdateTime(LocalDateTime.now());
                currTaskServiceBiz.updateById(currTask);

                //异步调用车辆
                // TODO: 2021/12/13 传入车辆id
                asynRunTaskService.asynRunTaskToWarehouse(currTask);
                return R.ok("操作成功！当前目标窑炉已满，托盘加入缓存区待加工队列。");
            }
            return R.failed("当前目标窑炉已满！缓存区已满！请稍后重试!");
        }
        return null;
    }

    /**
     * 加工完成、出炉。传入窑炉id，查询正在该窑炉加工的currTask,查询是否有空闲车辆，有，
     * （查询是否为自动任务，是，查询下一步的窑炉是否空闲，是，调车去下一窑炉加工，否，暂存缓存区）
     * 呼叫车辆暂存缓存区，车辆调度任务结束后，（结束后再查询缓存区队列避免车辆路线冲突）查询该窑炉的缓存区队列，开始下一托盘货物调度进窑炉
     * @return
     */
    public R runTaskForOutKiln(){
        return null;
    }
}
